@@ -1,70 +1,54 @@
-# -*- coding: utf-8 -*-
-
-from __future__ import absolute_import, division, unicode_literals
-
 import itertools
 import os
 import re
+from collections import OrderedDict
 
 from django import forms
 from django.conf import settings as django_settings
-from django.conf.urls import url
 from django.contrib import messages
 from django.contrib.admin import helpers
+from django.contrib.admin.utils import capfirst, quote, unquote
 from django.core.exceptions import PermissionDenied, ValidationError
 from django.core.paginator import EmptyPage, PageNotAnInteger, Paginator
-from django.core.urlresolvers import reverse
 from django.db import models, router
 from django.http import HttpResponse, HttpResponseRedirect
 from django.shortcuts import get_object_or_404, render
-from django.utils.encoding import force_text
-from django.utils.html import escape
-from django.utils.http import urlquote
+from django.urls import re_path, reverse
+from django.utils.encoding import force_str
+from django.utils.html import escape, format_html
+from django.utils.http import urlquote, urlunquote
 from django.utils.safestring import mark_safe
-from django.utils.translation import ugettext as _
-from django.utils.translation import ugettext_lazy, ungettext
+from django.utils.translation import gettext as _
+from django.utils.translation import gettext_lazy, ungettext
 
-from . import views
 from .. import settings
 from ..models import (
-    File,
-    Folder,
-    FolderPermission,
-    FolderRoot,
-    Image,
-    ImagesWithMissingData,
-    UnsortedImages,
-    tools,
+    File, Folder, FolderPermission, FolderRoot, ImagesWithMissingData,
+    UnsortedImages, tools,
 )
-from ..settings import FILER_PAGINATE_BY
+from ..settings import FILER_IMAGE_MODEL, FILER_PAGINATE_BY
 from ..thumbnail_processors import normalize_subject_location
-from ..utils.compatibility import (
-    capfirst,
-    get_delete_permission,
-    quote,
-    unquote,
-)
+from ..utils.compatibility import get_delete_permission
 from ..utils.filer_easy_thumbnails import FilerActionThumbnailer
+from ..utils.loader import load_model
+from . import views
 from .forms import CopyFilesAndFoldersForm, RenameFilesForm, ResizeImagesForm
 from .patched.admin_utils import get_deleted_objects
 from .permissions import PrimitivePermissionAwareModelAdmin
 from .tools import (
-    AdminContext,
-    admin_each_context,
-    admin_url_params_encoded,
-    check_files_edit_permissions,
-    check_files_read_permissions,
-    check_folder_edit_permissions,
-    check_folder_read_permissions,
-    popup_status,
-    userperms_for_request,
+    AdminContext, admin_url_params_encoded, check_files_edit_permissions,
+    check_files_read_permissions, check_folder_edit_permissions,
+    check_folder_read_permissions, popup_status, userperms_for_request,
 )
 
 
+Image = load_model(FILER_IMAGE_MODEL)
+
+
 class AddFolderPopupForm(forms.ModelForm):
     folder = forms.HiddenInput()
 
-    class Meta(object):
+    class Meta:
         model = Folder
         fields = ('name',)
 
@@ -74,16 +58,15 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
     exclude = ('parent',)
     list_per_page = 20
     list_filter = ('owner',)
-    search_fields = ['name', ]
-    raw_id_fields = ('owner',)
+    search_fields = ['name']
+    autocomplete_fields = ['owner']
     save_as = True  # see ImageAdmin
-    actions = ['files_set_public', 'files_set_private',
-               'delete_files_or_folders', 'move_files_and_folders',
+    actions = ['delete_files_or_folders', 'move_files_and_folders',
                'copy_files_and_folders', 'resize_images', 'rename_files']
 
     directory_listing_template = 'admin/filer/folder/directory_listing.html'
-    order_by_file_fields = ('_file_size', 'original_filename', 'name', 'owner',
-                            'uploaded_at', 'modified_at')
+    order_by_file_fields = ['_file_size', 'original_filename', 'name', 'owner',
+                            'uploaded_at', 'modified_at']
 
     def get_form(self, request, obj=None, **kwargs):
         """
@@ -96,12 +79,12 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
         if parent_id:
             return AddFolderPopupForm
         else:
-            folder_form = super(FolderAdmin, self).get_form(
+            folder_form = super().get_form(
                 request, obj=None, **kwargs)
 
             def folder_form_clean(form_obj):
                 cleaned_data = form_obj.cleaned_data
-                folders_with_same_name = Folder.objects.filter(
+                folders_with_same_name = self.get_queryset(request).filter(
                     parent=form_obj.instance.parent,
                     name=cleaned_data['name'])
                 if form_obj.instance.pk:
@@ -126,7 +109,7 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
         if not parent_id:
             parent_id = request.POST.get('parent_id', None)
         if parent_id:
-            parent = Folder.objects.get(id=parent_id)
+            parent = self.get_queryset(request).get(id=parent_id)
             r.parent = parent
         return r
 
@@ -136,11 +119,12 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
         instead of the default change_list_view
         """
         if (
-            request.POST and
-            '_continue' not in request.POST and
-            '_saveasnew' not in request.POST and
-            '_addanother' not in request.POST
+            request.POST
+            and '_continue' not in request.POST
+            and '_saveasnew' not in request.POST
+            and '_addanother' not in request.POST
         ):
+
             if obj.parent:
                 url = reverse('admin:filer-directory_listing',
                               kwargs={'folder_id': obj.parent.id})
@@ -151,15 +135,17 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
                 admin_url_params_encoded(request),
             )
             return HttpResponseRedirect(url)
-        return super(FolderAdmin, self).response_change(request, obj)
+        return super().response_change(request, obj)
 
     def render_change_form(self, request, context, add=False, change=False,
                            form_url='', obj=None):
+        info = self.model._meta.app_label, self.model._meta.model_name
         extra_context = {'show_delete': True,
+                         'history_url': 'admin:%s_%s_history' % info,
                          'is_popup': popup_status(request),
                          'filer_admin_context': AdminContext(request)}
         context.update(extra_context)
-        return super(FolderAdmin, self).render_change_form(
+        return super().render_change_form(
             request=request, context=context, add=add,
             change=change, form_url=form_url, obj=obj)
 
@@ -190,7 +176,7 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
             self.delete_files_or_folders(
                 request,
                 files_queryset=File.objects.none(),
-                folders_queryset=Folder.objects.filter(id=object_id)
+                folders_queryset=self.get_queryset(request).filter(id=object_id)
             )
             if parent_folder:
                 url = reverse('admin:filer-directory_listing',
@@ -206,48 +192,47 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
         return self.delete_files_or_folders(
             request,
             files_queryset=File.objects.none(),
-            folders_queryset=Folder.objects.filter(id=object_id)
+            folders_queryset=self.get_queryset(request).filter(id=object_id)
         )
 
     def icon_img(self, xs):
-        return mark_safe(('<img src="%simg/icons/plainfolder_32x32.png" '
-                          'alt="Folder Icon" />') % django_settings.MEDIA_ROOT)
+        return format_html('<img src="filer/icons/folder.svg" alt="Folder Icon" />', django_settings.STATIC_ROOT)
     icon_img.allow_tags = True
 
     def get_urls(self):
         return [
             # we override the default list view with our own directory listing
             # of the root directories
-            url(r'^$',
-                self.admin_site.admin_view(self.directory_listing),
-                name='filer-directory_listing-root'),
+            re_path(r'^$',
+                    self.admin_site.admin_view(self.directory_listing),
+                    name='filer-directory_listing-root'),
 
-            url(r'^last/$',
-                self.admin_site.admin_view(self.directory_listing),
-                {'viewtype': 'last'},
-                name='filer-directory_listing-last'),
+            re_path(r'^last/$',
+                    self.admin_site.admin_view(self.directory_listing),
+                    {'viewtype': 'last'},
+                    name='filer-directory_listing-last'),
 
-            url(r'^(?P<folder_id>\d+)/list/$',
-                self.admin_site.admin_view(self.directory_listing),
-                name='filer-directory_listing'),
+            re_path(r'^(?P<folder_id>\d+)/list/$',
+                    self.admin_site.admin_view(self.directory_listing),
+                    name='filer-directory_listing'),
 
-            url(r'^(?P<folder_id>\d+)/make_folder/$',
-                self.admin_site.admin_view(views.make_folder),
-                name='filer-directory_listing-make_folder'),
-            url(r'^make_folder/$',
-                self.admin_site.admin_view(views.make_folder),
-                name='filer-directory_listing-make_root_folder'),
+            re_path(r'^(?P<folder_id>\d+)/make_folder/$',
+                    self.admin_site.admin_view(views.make_folder),
+                    name='filer-directory_listing-make_folder'),
+            re_path(r'^make_folder/$',
+                    self.admin_site.admin_view(views.make_folder),
+                    name='filer-directory_listing-make_root_folder'),
 
-            url(r'^images_with_missing_data/$',
-                self.admin_site.admin_view(self.directory_listing),
-                {'viewtype': 'images_with_missing_data'},
-                name='filer-directory_listing-images_with_missing_data'),
+            re_path(r'^images_with_missing_data/$',
+                    self.admin_site.admin_view(self.directory_listing),
+                    {'viewtype': 'images_with_missing_data'},
+                    name='filer-directory_listing-images_with_missing_data'),
 
-            url(r'^unfiled_images/$',
-                self.admin_site.admin_view(self.directory_listing),
-                {'viewtype': 'unfiled_images'},
-                name='filer-directory_listing-unfiled_images'),
-        ] + super(FolderAdmin, self).get_urls()
+            re_path(r'^unfiled_images/$',
+                    self.admin_site.admin_view(self.directory_listing),
+                    {'viewtype': 'unfiled_images'},
+                    name='filer-directory_listing-unfiled_images'),
+        ] + super().get_urls()
 
     # custom views
     def directory_listing(self, request, folder_id=None, viewtype=None):
@@ -259,8 +244,8 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
         elif viewtype == 'last':
             last_folder_id = request.session.get('filer_last_folder_id')
             try:
-                Folder.objects.get(id=last_folder_id)
-            except Folder.DoesNotExist:
+                self.get_queryset(request).get(id=last_folder_id)
+            except self.model.DoesNotExist:
                 url = reverse('admin:filer-directory_listing-root')
                 url = "%s%s" % (url, admin_url_params_encoded(request))
             else:
@@ -270,7 +255,7 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
         elif folder_id is None:
             folder = FolderRoot()
         else:
-            folder = get_object_or_404(Folder, id=folder_id)
+            folder = get_object_or_404(self.get_queryset(request), id=folder_id)
         request.session['filer_last_folder_id'] = folder_id
 
         # Check actions to see if any are available on this changelist
@@ -285,9 +270,9 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
                 pass
 
         # search
-        q = request.GET.get('q', None)
+        q = request.GET.get('q')
         if q:
-            search_terms = unquote(q).split(" ")
+            search_terms = urlunquote(q).split(' ')
             search_mode = True
         else:
             search_terms = []
@@ -306,7 +291,7 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
                 file_qs = File.objects.filter(
                     folder__in=folder.get_descendants(include_self=True))
             else:
-                folder_qs = Folder.objects.all()
+                folder_qs = self.get_queryset(request)
                 file_qs = File.objects.all()
             folder_qs = self.filter_folder(folder_qs, search_terms)
             file_qs = self.filter_file(file_qs, search_terms)
@@ -336,7 +321,11 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
         perms = FolderPermission.objects.get_read_id_list(request.user)
         root_exclude_kw = {'parent__isnull': False, 'parent__id__in': perms}
         if perms != 'All':
-            file_qs = file_qs.filter(models.Q(folder__id__in=perms) | models.Q(owner=request.user))
+            file_qs = file_qs.filter(
+                models.Q(folder__id__in=perms)
+                | models.Q(folder_id__isnull=True)
+                | models.Q(owner=request.user)
+            )
             folder_qs = folder_qs.filter(models.Q(id__in=perms) | models.Q(owner=request.user))
         else:
             root_exclude_kw.pop('parent__id__in')
@@ -353,15 +342,14 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
                 'has_add_children_permission':
                     folder.has_add_children_permission(request),
             }
-        except:
+        except:  # noqa
             permissions = {}
 
         if order_by is None or len(order_by) == 0:
             folder_files.sort()
 
         items = folder_children + folder_files
-        items_permissions = [(item, {'change': self.has_change_permission(request, item)}) for item in items]
-        paginator = Paginator(items_permissions, FILER_PAGINATE_BY)
+        paginator = Paginator(items, FILER_PAGINATE_BY)
 
         # Are we moving to clipboard?
         if request.method == 'POST' and '_save' not in request.POST:
@@ -377,8 +365,11 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
 
         selected = request.POST.getlist(helpers.ACTION_CHECKBOX_NAME)
         # Actions with no confirmation
-        if (actions and request.method == 'POST' and
-                'index' in request.POST and '_save' not in request.POST):
+        if (
+            actions and request.method == 'POST'
+            and 'index' in request.POST
+            and '_save' not in request.POST
+        ):
             if selected:
                 response = self.response_action(request, files_queryset=file_qs, folders_queryset=folder_qs)
                 if response:
@@ -389,9 +380,12 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
                 self.message_user(request, msg)
 
         # Actions with confirmation
-        if (actions and request.method == 'POST' and
-                helpers.ACTION_CHECKBOX_NAME in request.POST and
-                'index' not in request.POST and '_save' not in request.POST):
+        if (
+            actions and request.method == 'POST'
+            and helpers.ACTION_CHECKBOX_NAME in request.POST
+            and 'index' not in request.POST
+            and '_save' not in request.POST
+        ):
             if selected:
                 response = self.response_action(request, files_queryset=file_qs, folders_queryset=folder_qs)
                 if response:
@@ -415,20 +409,20 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
         except EmptyPage:
             paginated_items = paginator.page(paginator.num_pages)
 
-        context = admin_each_context(self.admin_site, request)
+        context = self.admin_site.each_context(request)
         context.update({
             'folder': folder,
             'clipboard_files': File.objects.filter(
                 in_clipboards__clipboarditem__clipboard__user=request.user
             ).distinct(),
             'paginator': paginator,
-            'paginated_items': paginated_items,  # [(item, item_perms), ]
+            'paginated_items': paginated_items,
             'virtual_items': virtual_items,
             'uploader_connections': settings.FILER_UPLOADER_CONNECTIONS,
             'permissions': permissions,
             'permstest': userperms_for_request(folder, request),
             'current_url': request.path,
-            'title': 'Directory listing for %s' % folder.name,
+            'title': _('Directory listing for %(folder_name)s') % {'folder_name': folder.name},
             'search_string': ' '.join(search_terms),
             'q': urlquote(q),
             'show_result_count': show_result_count,
@@ -474,9 +468,9 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
 
     def filter_file(self, qs, terms=()):
         for term in terms:
-            filters = (models.Q(name__icontains=term) |
-                       models.Q(description__icontains=term) |
-                       models.Q(original_filename__icontains=term))
+            filters = (models.Q(name__icontains=term)
+                       | models.Q(description__icontains=term)
+                       | models.Q(original_filename__icontains=term))
             for filter_ in self.get_owner_filter_lookups():
                 filters |= models.Q(**{filter_: term})
             qs = qs.filter(filters)
@@ -489,12 +483,9 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
         User model.  For the built-in User model, that means username,
         first_name, last_name, and email.
         """
-        try:
-            from django.contrib.auth import get_user_model
-        except ImportError:  # Django < 1.5
-            from django.contrib.auth.models import User
-        else:
-            User = get_user_model()
+        from django.contrib.auth import get_user_model
+        User = get_user_model()
+
         return [
             field.name for field in User._meta.fields
             if isinstance(field, models.CharField) and field.name != 'password'
@@ -586,7 +577,14 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
             return None
 
     def get_actions(self, request):
-        actions = super(FolderAdmin, self).get_actions(request)
+        if settings.FILER_ENABLE_PERMISSIONS:
+            actions = OrderedDict()
+            actions['files_set_public'] = self.get_action('files_set_public')
+            actions['files_set_private'] = self.get_action('files_set_private')
+            actions.update(super().get_actions(request))
+        else:
+            actions = super().get_actions(request)
+
         if 'delete_selected' in actions:
             del actions['delete_selected']
         return actions
@@ -631,7 +629,7 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
 
         return None
 
-    move_to_clipboard.short_description = ugettext_lazy(
+    move_to_clipboard.short_description = gettext_lazy(
         "Move selected files to clipboard")
 
     def files_set_public_or_private(self, request, set_public, files_queryset,
@@ -644,7 +642,9 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
         if not self.has_change_permission(request):
             raise PermissionDenied
 
-        if request.method != 'POST':
+        permissions_enabled = settings.FILER_ENABLE_PERMISSIONS
+
+        if request.method != 'POST' or not permissions_enabled:
             return None
 
         check_files_edit_permissions(request, files_queryset)
@@ -680,14 +680,14 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
         return self.files_set_public_or_private(request, False, files_queryset,
                                                 folders_queryset)
 
-    files_set_private.short_description = ugettext_lazy(
+    files_set_private.short_description = gettext_lazy(
         "Enable permissions for selected files")
 
     def files_set_public(self, request, files_queryset, folders_queryset):
         return self.files_set_public_or_private(request, True, files_queryset,
                                                 folders_queryset)
 
-    files_set_public.short_description = ugettext_lazy(
+    files_set_public.short_description = gettext_lazy(
         "Disable permissions for selected files")
 
     def delete_files_or_folders(self, request, files_queryset, folders_queryset):
@@ -735,7 +735,7 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
             if n:
                 # delete all explicitly selected files
                 for f in files_queryset:
-                    self.log_deletion(request, f, force_text(f))
+                    self.log_deletion(request, f, force_str(f))
                     f.delete()
                 # delete all files in all selected folders and their children
                 # This would happen automatically by ways of the delete
@@ -747,11 +747,11 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
                     folder_ids.update(
                         folder.get_descendants().values_list('id', flat=True))
                 for f in File.objects.filter(folder__in=folder_ids):
-                    self.log_deletion(request, f, force_text(f))
+                    self.log_deletion(request, f, force_str(f))
                     f.delete()
                 # delete all folders
                 for f in folders_queryset:
-                    self.log_deletion(request, f, force_text(f))
+                    self.log_deletion(request, f, force_str(f))
                     f.delete()
                 self.message_user(request, _("Successfully deleted %(count)d files and/or folders.") % {"count": n, })
             # Return None to display the change list page again.
@@ -762,7 +762,7 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
         else:
             title = _("Are you sure?")
 
-        context = admin_each_context(self.admin_site, request)
+        context = self.admin_site.each_context(request)
         context.update({
             "title": title,
             "instance": current_folder,
@@ -787,7 +787,7 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
             context
         )
 
-    delete_files_or_folders.short_description = ugettext_lazy(
+    delete_files_or_folders.short_description = gettext_lazy(
         "Delete selected files and/or folders")
 
     # Copied from django.contrib.admin.util
@@ -804,14 +804,11 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
             if not user.has_perm(p):
                 perms_needed.add(opts.verbose_name)
             # Display a link to the admin page.
-            return mark_safe('%s: <a href="%s">%s</a>' %
-                             (escape(capfirst(opts.verbose_name)),
-                              admin_url,
-                              escape(obj)))
+            return format_html('{}: <a href="{}">{}</a>', escape(capfirst(opts.verbose_name)), admin_url, escape(obj))
         else:
             # Don't display link to edit, because it either has no
             # admin or is edited inline.
-            return '%s: %s' % (capfirst(opts.verbose_name), force_text(obj))
+            return '%s: %s' % (capfirst(opts.verbose_name), force_str(obj))
 
     def _check_copy_perms(self, request, files_queryset, folders_queryset):
         try:
@@ -864,12 +861,12 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
 
             # We do not allow copying/moving back to the folder itself
             enabled = (allow_self or fo != current_folder) and fo.has_add_children_permission(request)
-            yield (fo, (mark_safe(("&nbsp;&nbsp;" * level) + force_text(fo)), enabled))
+            yield (fo, (mark_safe(("&nbsp;&nbsp;" * level) + force_str(fo)), enabled))
             for c in self._list_all_destination_folders_recursive(request, folders_queryset, current_folder, fo.children.all(), allow_self, level + 1):
                 yield c
 
     def _list_all_destination_folders(self, request, folders_queryset, current_folder, allow_self):
-        root_folders = Folder.objects.filter(parent__isnull=True).order_by('name')
+        root_folders = self.get_queryset(request).filter(parent__isnull=True).order_by('name')
         return list(self._list_all_destination_folders_recursive(request, folders_queryset, current_folder, root_folders, allow_self, 0))
 
     def _move_files_and_folders_impl(self, files_queryset, folders_queryset, destination):
@@ -893,15 +890,15 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
             if perms_needed:
                 raise PermissionDenied
             try:
-                destination = Folder.objects.get(pk=request.POST.get('destination'))
-            except Folder.DoesNotExist:
+                destination = self.get_queryset(request).get(pk=request.POST.get('destination'))
+            except self.model.DoesNotExist:
                 raise PermissionDenied
             folders_dict = dict(folders)
             if destination not in folders_dict or not folders_dict[destination][1]:
                 raise PermissionDenied
             # We count only topmost files and folders here
             n = files_queryset.count() + folders_queryset.count()
-            conflicting_names = [folder.name for folder in Folder.objects.filter(parent=destination, name__in=folders_queryset.values('name'))]
+            conflicting_names = [folder.name for folder in self.get_queryset(request).filter(parent=destination, name__in=folders_queryset.values('name'))]
             if conflicting_names:
                 messages.error(request, _("Folders with names %s already exist at the selected "
                                           "destination") % ", ".join(conflicting_names))
@@ -913,7 +910,7 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
                 })
             return None
 
-        context = admin_each_context(self.admin_site, request)
+        context = self.admin_site.each_context(request)
         context.update({
             "title": _("Move files and/or folders"),
             "instance": current_folder,
@@ -932,7 +929,7 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
         # Display the destination folder selection page
         return render(request, "admin/filer/folder/choose_move_destination.html", context)
 
-    move_files_and_folders.short_description = ugettext_lazy("Move selected files and/or folders")
+    move_files_and_folders.short_description = gettext_lazy("Move selected files and/or folders")
 
     def _rename_file(self, file_obj, form_data, counter, global_counter):
         original_basename, original_extension = os.path.splitext(file_obj.original_filename)
@@ -996,7 +993,7 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
         else:
             form = RenameFilesForm()
 
-        context = admin_each_context(self.admin_site, request)
+        context = self.admin_site.each_context(request)
         context.update({
             "title": _("Rename files"),
             "instance": current_folder,
@@ -1015,7 +1012,7 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
         # Display the rename format selection page
         return render(request, "admin/filer/folder/choose_rename_format.html", context)
 
-    rename_files.short_description = ugettext_lazy("Rename files")
+    rename_files.short_description = gettext_lazy("Rename files")
 
     def _generate_new_filename(self, filename, suffix):
         basename, extension = os.path.splitext(filename)
@@ -1035,6 +1032,7 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
         file_obj.id = None
         file_obj.save()
         file_obj.folder = destination
+        file_obj._file_data_changed_hint = False  # no need to update size, sha1, etc.
         file_obj.file = file_obj._copy_file(filename)
         file_obj.original_filename = self._generate_new_filename(file_obj.original_filename, suffix)
         file_obj.save()
@@ -1100,14 +1098,14 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
             form = CopyFilesAndFoldersForm(request.POST)
             if form.is_valid():
                 try:
-                    destination = Folder.objects.get(pk=request.POST.get('destination'))
-                except Folder.DoesNotExist:
+                    destination = self.get_queryset(request).get(pk=request.POST.get('destination'))
+                except self.model.DoesNotExist:
                     raise PermissionDenied
                 folders_dict = dict(folders)
                 if destination not in folders_dict or not folders_dict[destination][1]:
                     raise PermissionDenied
                 if files_queryset.count() + folders_queryset.count():
-                    # We count all files and folders here (recursivelly)
+                    # We count all files and folders here (recursively)
                     n = self._copy_files_and_folders_impl(files_queryset, folders_queryset, destination, form.cleaned_data['suffix'], False)
                     self.message_user(request, _("Successfully copied %(count)d files and/or folders to folder '%(destination)s'.") % {
                         "count": n,
@@ -1125,7 +1123,7 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
             else:
                 selected_destination_folder = 0
 
-        context = admin_each_context(self.admin_site, request)
+        context = self.admin_site.each_context(request)
         context.update({
             "title": _("Copy files and/or folders"),
             "instance": current_folder,
@@ -1146,7 +1144,7 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
         # Display the destination folder selection page
         return render(request, "admin/filer/folder/choose_copy_destination.html", context)
 
-    copy_files_and_folders.short_description = ugettext_lazy("Copy selected files and/or folders")
+    copy_files_and_folders.short_description = gettext_lazy("Copy selected files and/or folders")
 
     def _check_resize_perms(self, request, files_queryset, folders_queryset):
         try:
@@ -1186,7 +1184,7 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
     def _resize_image(self, image, form_data):
         original_width = float(image.width)
         original_height = float(image.height)
-        thumbnailer = FilerActionThumbnailer(file=image.file.file, name=image.file.name, source_storage=image.file.source_storage, thumbnail_storage=image.file.source_storage)
+        thumbnailer = FilerActionThumbnailer(file=image.file, name=image.file.name, source_storage=image.file.source_storage, thumbnail_storage=image.file.source_storage)
         # This should overwrite the original image
         new_image = thumbnailer.get_thumbnail({
             'size': tuple(int(form_data[d] or 0) for d in ('width', 'height')),
@@ -1195,8 +1193,10 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
             'subject_location': image.subject_location,
         })
         image.file.file = new_image.file
-        image.generate_sha1()
-        image.save()  # Also gets new width and height
+        # Since only file data was changed, there is no way for file field to know about the change.
+        # To update size, sha1, width and height fields let's call file_data_changed callback directly.
+        image.file_data_changed()
+        image.save()
 
         subject_location = normalize_subject_location(image.subject_location)
         if subject_location:
@@ -1247,14 +1247,14 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
                     form.cleaned_data['crop'] = form.cleaned_data['thumbnail_option'].crop
                     form.cleaned_data['upscale'] = form.cleaned_data['thumbnail_option'].upscale
                 if files_queryset.count() + folders_queryset.count():
-                    # We count all files here (recursivelly)
+                    # We count all files here (recursively)
                     n = self._resize_images_impl(files_queryset, folders_queryset, form.cleaned_data)
                     self.message_user(request, _("Successfully resized %(count)d images.") % {"count": n, })
                 return None
         else:
             form = ResizeImagesForm()
 
-        context = admin_each_context(self.admin_site, request)
+        context = self.admin_site.each_context(request)
         context.update({
             "title": _("Resize images"),
             "instance": current_folder,
@@ -1274,4 +1274,4 @@ class FolderAdmin(PrimitivePermissionAwareModelAdmin):
         # Display the resize options page
         return render(request, "admin/filer/folder/choose_images_resize_options.html", context)
 
-    resize_images.short_description = ugettext_lazy("Resize selected images")
+    resize_images.short_description = gettext_lazy("Resize selected images")
