@@ -1,6 +1,3 @@
-# -*- coding: utf-8 -*-
-from __future__ import absolute_import
-
 import os
 import stat
 
@@ -8,7 +5,6 @@ from django.http import Http404, HttpResponse, HttpResponseNotModified
 from django.utils.http import http_date
 from django.views.static import was_modified_since
 
-from ...utils.compatibility import LTE_DJANGO_1_4
 from .base import ServerBase
 
 
@@ -19,21 +15,19 @@ class DefaultServer(ServerBase):
 
     This will only work for files that can be accessed in the local filesystem.
     """
-    def serve(self, request, file_obj, **kwargs):
-        fullpath = file_obj.path
+    def serve(self, request, filer_file, **kwargs):
+        fullpath = filer_file.path
         # the following code is largely borrowed from `django.views.static.serve`
         # and django-filetransfers: filetransfers.backends.default
         if not os.path.exists(fullpath):
             raise Http404('"%s" does not exist' % fullpath)
         # Respect the If-Modified-Since header.
         statobj = os.stat(fullpath)
-
-        content_type_key = 'mimetype' if LTE_DJANGO_1_4 else 'content_type'
-        response_params = {content_type_key: self.get_mimetype(fullpath)}
+        response_params = {'content_type': filer_file.mime_type}
         if not was_modified_since(request.META.get('HTTP_IF_MODIFIED_SINCE'),
                                   statobj[stat.ST_MTIME], statobj[stat.ST_SIZE]):
             return HttpResponseNotModified(**response_params)
         response = HttpResponse(open(fullpath, 'rb').read(), **response_params)
         response["Last-Modified"] = http_date(statobj[stat.ST_MTIME])
-        self.default_headers(request=request, response=response, file_obj=file_obj, **kwargs)
+        self.default_headers(request=request, response=response, file_obj=filer_file.file, **kwargs)
         return response
