@@ -5,7 +5,7 @@ from django.http.multipartparser import (
     ChunkIter, SkipFile, StopFutureHandlers, StopUpload, exhaust,
 )
 from django.template.defaultfilters import slugify as slugify_django
-from django.utils.encoding import force_text
+from django.utils.encoding import force_str
 from django.utils.text import get_valid_filename as get_valid_filename_django
 
 from unidecode import unidecode
@@ -114,14 +114,15 @@ def handle_request_files_upload(request):
     filename = upload.name
     _, iext = os.path.splitext(filename)
     mime_type = upload.content_type.lower()
-    if iext not in mimetypes.guess_all_extensions(mime_type):
+    extensions = mimetypes.guess_all_extensions(mime_type)
+    if mime_type != 'application/octet-stream' and extensions and iext.lower() not in extensions:
         msg = "MIME-Type '{mimetype}' does not correspond to file extension of {filename}."
         raise UploadException(msg.format(mimetype=mime_type, filename=filename))
     return upload, filename, is_raw, mime_type
 
 
 def slugify(string):
-    return slugify_django(unidecode(force_text(string)))
+    return slugify_django(unidecode(force_str(string)))
 
 
 def get_valid_filename(s):
